@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
 
 *******************************************************
 
@@ -36,7 +36,7 @@ permission notice:
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
 
 *******************************************************/
 
@@ -178,8 +178,7 @@ xb_write_galera_info(bool incremental_prepare)
 		return;
 	}
 
-	memset(&xid, 0, sizeof(xid));
-	xid.formatID = -1;
+	xid.null();
 
 	if (!trx_sys_read_wsrep_checkpoint(&xid)) {
 
@@ -194,7 +193,7 @@ xb_write_galera_info(bool incremental_prepare)
 	fp = fopen(XB_GALERA_INFO_FILENAME, "w");
 	if (fp == NULL) {
 
-		msg("mariabackup: error: "
+		die(
 		    "could not create " XB_GALERA_INFO_FILENAME
 		    ", errno = %d\n",
 		    errno);
@@ -208,11 +207,10 @@ xb_write_galera_info(bool incremental_prepare)
 
 	if (fprintf(fp, "%s:%lld", uuid_str, (long long) seqno) < 0) {
 
-		msg("mariabackup: error: "
+		die(
 		    "could not write to " XB_GALERA_INFO_FILENAME
 		    ", errno = %d\n",
-		    errno);
-		exit(EXIT_FAILURE);
+		    errno);;
 	}
 
 	fclose(fp);
