@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1335  USA */
 
 #include <my_global.h>
 #include "sql_priv.h"
@@ -222,13 +222,14 @@ public:
     return str;
   }
   enum Item_result result_type () const { return STRING_RESULT; }
-  void fix_length_and_dec()
+  bool fix_length_and_dec()
   {
     max_length= MAX_BLOB_WIDTH;
     collation.collation= pxml->charset();
     // To avoid premature evaluation, mark all nodeset functions as non-const.
     used_tables_cache= RAND_TABLE_BIT;
     const_item_cache= false;
+    return FALSE;
   }
   const char *func_name() const { return "nodeset"; }
   bool check_vcol_func_processor(void *arg)
@@ -456,7 +457,7 @@ public:
     Item_nodeset_func(thd, pxml), string_cache(str_arg) { }
   String *val_nodeset(String *res)
   { return string_cache; }
-  void fix_length_and_dec() { max_length= MAX_BLOB_WIDTH; }
+  bool fix_length_and_dec() { max_length= MAX_BLOB_WIDTH; return FALSE; }
   Item *get_copy(THD *thd, MEM_ROOT *mem_root)
   { return get_item_copy<Item_nodeset_context_cache>(thd, mem_root, this); }
 };
@@ -470,7 +471,7 @@ public:
   Item_func_xpath_position(THD *thd, Item *a, String *p):
     Item_int_func(thd, a), pxml(p) {}
   const char *func_name() const { return "xpath_position"; }
-  void fix_length_and_dec() { max_length=10; }
+  bool fix_length_and_dec() { max_length=10; return FALSE; }
   longlong val_int()
   {
     String *flt= args[0]->val_nodeset(&tmp_value);
@@ -491,7 +492,7 @@ public:
   Item_func_xpath_count(THD *thd, Item *a, String *p):
     Item_int_func(thd, a), pxml(p) {}
   const char *func_name() const { return "xpath_count"; }
-  void fix_length_and_dec() { max_length=10; }
+  bool fix_length_and_dec() { max_length=10; return FALSE; }
   longlong val_int()
   {
     uint predicate_supplied_context_size;
@@ -2709,10 +2710,10 @@ my_xpath_parse(MY_XPATH *xpath, const char *str, const char *strend)
 }
 
 
-void Item_xml_str_func::fix_length_and_dec()
+bool Item_xml_str_func::fix_length_and_dec()
 {
   max_length= MAX_BLOB_WIDTH;
-  agg_arg_charsets_for_comparison(collation, args, arg_count);
+  return agg_arg_charsets_for_comparison(collation, args, arg_count);
 }
 
 
