@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA */
 
 /**
   @file
@@ -243,6 +243,12 @@ protected:
   uchar *global_var_ptr()
   { return ((uchar*)&global_system_variables) + offset; }
 
+  void *max_var_ptr()
+  {
+    return scope() == SESSION ? (((uchar*)&max_system_variables) + offset) :
+                                0;
+  }
+
   friend class Session_sysvars_tracker;
   friend class Session_tracker;
 };
@@ -399,6 +405,16 @@ int sql_set_variables(THD *thd, List<set_var_base> *var_list, bool free);
     set_sys_var_value_origin(&VAR, sys_var::AUTO);      \
   } while(0)
 
+#define SYSVAR_AUTOSIZE_IF_CHANGED(VAR,VAL,TYPE)        \
+  do {                                                  \
+    TYPE tmp= (VAL);                                    \
+    if (VAR != tmp)                                     \
+    {                                                   \
+      VAR= (VAL);                                       \
+      set_sys_var_value_origin(&VAR, sys_var::AUTO);    \
+    }                                                   \
+  } while(0)
+
 void set_sys_var_value_origin(void *ptr, enum sys_var::where here);
 
 enum sys_var::where get_sys_var_value_origin(void *ptr);
@@ -414,7 +430,9 @@ sql_mode_t expand_sql_mode(sql_mode_t sql_mode);
 bool sql_mode_string_representation(THD *thd, sql_mode_t sql_mode, LEX_STRING *ls);
 int default_regex_flags_pcre(const THD *thd);
 
-extern sys_var *Sys_autocommit_ptr;
+extern sys_var *Sys_autocommit_ptr, *Sys_last_gtid_ptr,
+  *Sys_character_set_client_ptr, *Sys_character_set_connection_ptr,
+  *Sys_character_set_results_ptr;
 
 CHARSET_INFO *get_old_charset_by_name(const char *old_name);
 
@@ -422,6 +440,7 @@ int sys_var_init();
 uint sys_var_elements();
 int sys_var_add_options(DYNAMIC_ARRAY *long_options, int parse_flags);
 void sys_var_end(void);
+bool check_has_super(sys_var *self, THD *thd, set_var *var);
 
 #endif
 
