@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2016, 2018, MariaDB Corporation.
+Copyright (c) 2016, 2019, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -12,7 +12,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -27,11 +27,8 @@ Created 25/08/2016 Jan Lindström
 #include "dict0stats.h"
 #include "dict0stats_bg.h"
 #include "dict0defrag_bg.h"
-#include "row0mysql.h"
+#include "btr0btr.h"
 #include "srv0start.h"
-#include "ut0new.h"
-
-#include <vector>
 
 static ib_mutex_t		defrag_pool_mutex;
 
@@ -247,7 +244,7 @@ dict_stats_save_defrag_summary(
 		return DB_SUCCESS;
 	}
 
-	rw_lock_x_lock(dict_operation_lock);
+	rw_lock_x_lock(&dict_operation_lock);
 	mutex_enter(&dict_sys->mutex);
 
 	ret = dict_stats_save_index_stat(index, now, "n_pages_freed",
@@ -258,7 +255,7 @@ dict_stats_save_defrag_summary(
 					 NULL);
 
 	mutex_exit(&dict_sys->mutex);
-	rw_lock_x_unlock(dict_operation_lock);
+	rw_lock_x_unlock(&dict_operation_lock);
 
 	return (ret);
 }
@@ -298,7 +295,7 @@ dict_stats_save_defrag_stats(
 		return DB_SUCCESS;
 	}
 
-	rw_lock_x_lock(dict_operation_lock);
+	rw_lock_x_lock(&dict_operation_lock);
 
 	mutex_enter(&dict_sys->mutex);
 	ret = dict_stats_save_index_stat(index, now, "n_page_split",
@@ -331,7 +328,7 @@ dict_stats_save_defrag_stats(
 
 end:
 	mutex_exit(&dict_sys->mutex);
-	rw_lock_x_unlock(dict_operation_lock);
+	rw_lock_x_unlock(&dict_operation_lock);
 
 	return (ret);
 }
