@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -27,12 +27,10 @@ Created 5/7/1996 Heikki Tuuri
 #ifndef lock0lock_h
 #define lock0lock_h
 
-#include "univ.i"
 #include "buf0types.h"
 #include "trx0types.h"
 #include "mtr0types.h"
 #include "rem0types.h"
-#include "dict0types.h"
 #include "que0types.h"
 #include "lock0types.h"
 #include "hash0hash.h"
