@@ -12,7 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1335  USA */
 
 #include "myisamdef.h"
 #ifdef HAVE_SYS_MMAN_H
@@ -260,10 +260,12 @@ int mi_extra(MI_INFO *info, enum ha_extra_function function, void *extra_arg)
     break;
   case HA_EXTRA_PREPARE_FOR_DROP:
     /* Signals about intent to delete this table */
-    //share->deleting= TRUE;
+    share->deleting= TRUE;
     share->global_changed= FALSE;     /* force writing changed flag */
     _mi_mark_file_changed(info);
-    /* Fall through */
+    if (share->temporary)
+      break;
+    /* fall through */
   case HA_EXTRA_PREPARE_FOR_RENAME:
     DBUG_ASSERT(!share->temporary);
     mysql_mutex_lock(&THR_LOCK_myisam);
