@@ -2,7 +2,7 @@
 
 Copyright (c) 1995, 2015, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2008, Google Inc.
-Copyright (c) 2013, 2020, MariaDB Corporation.
+Copyright (c) 2013, 2015, MariaDB Corporation. All Rights Reserved.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -123,10 +123,8 @@ struct sync_cell_t {
 					has not been signalled in the
 					period between the reset and
 					wait call. */
-	/** time(NULL) when the wait cell was reserved.
-	FIXME: sync_array_print_long_waits_low() may display bogus
-	warnings when the system time is adjusted to the past! */
-	time_t		reservation_time;
+	time_t		reservation_time;/*!< time when the thread reserved
+					the wait cell */
 };
 
 /* NOTE: It is allowed for a thread to wait
@@ -414,7 +412,7 @@ sync_array_reserve_cell(
                         event = sync_cell_get_event(cell);
 			cell->signal_count = os_event_reset(event);
 
-			cell->reservation_time = time(NULL);
+			cell->reservation_time = ut_time();
 
 			cell->thread = os_thread_get_curr_id();
 
@@ -1164,7 +1162,7 @@ sync_array_print_long_waits(
 		sync_array_exit(arr);
 	}
 
-	if (noticed && srv_monitor_event) {
+	if (noticed) {
 		ibool	old_val;
 
 		fprintf(stderr,
